@@ -111,7 +111,7 @@ kerb5_init(struct passwd *pw, char **promptp, sudo_auth *auth)
     if ((error = krb5_parse_name(sudo_context, pw->pw_name,
 	&(sudo_krb5_data.princ)))) {
 	log_error(NO_EXIT|NO_MAIL,
-		  "%s: unable to parse '%s': %s", auth->name, pw->pw_name,
+		  _("%s: unable to parse '%s': %s"), auth->name, pw->pw_name,
 		  error_message(error));
 	return AUTH_FAILURE;
     }
@@ -124,7 +124,7 @@ kerb5_init(struct passwd *pw, char **promptp, sudo_auth *auth)
 #if 1
     if ((error = krb5_unparse_name(sudo_context, princ, &pname))) {
 	log_error(NO_EXIT|NO_MAIL,
-		  "%s: unable to unparse princ ('%s'): %s", auth->name,
+		  _("%s: unable to unparse princ ('%s'): %s"), auth->name,
 		  pw->pw_name, error_message(error));
 	return AUTH_FAILURE;
     }
@@ -141,7 +141,7 @@ kerb5_init(struct passwd *pw, char **promptp, sudo_auth *auth)
     if ((error = krb5_cc_resolve(sudo_context, cache_name,
 	&(sudo_krb5_data.ccache)))) {
 	log_error(NO_EXIT|NO_MAIL,
-		  "%s: unable to resolve ccache: %s", auth->name,
+		  _("%s: unable to resolve ccache: %s"), auth->name,
 		  error_message(error));
 	return AUTH_FAILURE;
     }
@@ -185,7 +185,7 @@ kerb5_verify(struct passwd *pw, char *pass, sudo_auth *auth)
     error = krb5_get_init_creds_opt_alloc(sudo_context, &opts);
     if (error) {
 	log_error(NO_EXIT|NO_MAIL,
-		  "%s: unable to allocate options: %s", auth->name,
+		  _("%s: unable to allocate options: %s"), auth->name,
 		  error_message(error));
 	goto done;
     }
@@ -201,7 +201,7 @@ kerb5_verify(struct passwd *pw, char *pass, sudo_auth *auth)
 	/* Don't print error if just a bad password */
 	if (error != KRB5KRB_AP_ERR_BAD_INTEGRITY)
 	    log_error(NO_EXIT|NO_MAIL,
-		      "%s: unable to get credentials: %s", auth->name,
+		      _("%s: unable to get credentials: %s"), auth->name,
 		      error_message(error));
 	goto done;
     }
@@ -214,11 +214,11 @@ kerb5_verify(struct passwd *pw, char *pass, sudo_auth *auth)
     /* Store cred in cred cache. */
     if ((error = krb5_cc_initialize(sudo_context, ccache, princ))) {
 	log_error(NO_EXIT|NO_MAIL,
-		  "%s: unable to initialize ccache: %s", auth->name,
+		  _("%s: unable to initialize ccache: %s"), auth->name,
 		  error_message(error));
     } else if ((error = krb5_cc_store_cred(sudo_context, ccache, creds))) {
 	log_error(NO_EXIT|NO_MAIL,
-		  "%s: unable to store cred in ccache: %s", auth->name,
+		  _("%s: unable to store cred in ccache: %s"), auth->name,
 		  error_message(error));
     }
 
@@ -281,7 +281,7 @@ verify_krb_v5_tgt(krb5_context sudo_context, krb5_creds *cred, char *auth_name)
     if ((error = krb5_sname_to_principal(sudo_context, NULL, NULL,
 					KRB5_NT_SRV_HST, &server))) {
 	log_error(NO_EXIT|NO_MAIL,
-		  "%s: unable to get host principal: %s", auth_name,
+		  _("%s: unable to get host principal: %s"), auth_name,
 		  error_message(error));
 	return -1;
     }
@@ -296,8 +296,8 @@ verify_krb_v5_tgt(krb5_context sudo_context, krb5_creds *cred, char *auth_name)
     krb5_free_principal(sudo_context, server);
     if (error)
 	log_error(NO_EXIT|NO_MAIL,
-		  "%s: Cannot verify TGT! Possible attack!: %s", auth_name,
-		  error_message(error));
+		  _("%s: Cannot verify TGT! Possible attack!: %s"),
+		  auth_name, error_message(error));
     return error;
 }
 #endif
