@@ -49,8 +49,7 @@
 # include <security/pam_appl.h>
 #endif
 
-#ifdef HAVE_DGETTEXT
-# include <libintl.h>
+#ifdef HAVE_LIBINTL_H
 # if defined(__LINUX_PAM__)
 #  define PAM_TEXT_DOMAIN	"Linux-PAM"
 # elif defined(__sun__)
@@ -97,7 +96,7 @@ pam_init(struct passwd *pw, char **promptp, sudo_auth *auth)
 #endif
 	pam_status = pam_start("sudo", pw->pw_name, &pam_conv, &pamh);
     if (pam_status != PAM_SUCCESS) {
-	log_error(USE_ERRNO|NO_EXIT|NO_MAIL, "unable to initialize PAM");
+	log_error(USE_ERRNO|NO_EXIT|NO_MAIL, _("unable to initialize PAM"));
 	return AUTH_FATAL;
     }
 
@@ -140,28 +139,27 @@ pam_verify(struct passwd *pw, char *prompt, sudo_auth *auth)
 		case PAM_SUCCESS:
 		    return AUTH_SUCCESS;
 		case PAM_AUTH_ERR:
-		    log_error(NO_EXIT|NO_MAIL,
-			"account validation failure, is your account locked?");
+		    log_error(NO_EXIT|NO_MAIL, _("account validation failure, "
+			"is your account locked?"));
 		    return AUTH_FATAL;
 		case PAM_NEW_AUTHTOK_REQD:
-		    log_error(NO_EXIT|NO_MAIL, "%s, %s",
-			"Account or password is expired",
-			"reset your password and try again");
+		    log_error(NO_EXIT|NO_MAIL, _("Account or password is "
+			"expired, reset your password and try again"));
 		    *pam_status = pam_chauthtok(pamh,
 			PAM_CHANGE_EXPIRED_AUTHTOK);
 		    if (*pam_status == PAM_SUCCESS)
 			return AUTH_SUCCESS;
 		    if ((s = pam_strerror(pamh, *pam_status)))
-			log_error(NO_EXIT|NO_MAIL, "pam_chauthtok: %s", s);
+			log_error(NO_EXIT|NO_MAIL, _("pam_chauthtok: %s"), s);
 		    return AUTH_FAILURE;
 		case PAM_AUTHTOK_EXPIRED:
 		    log_error(NO_EXIT|NO_MAIL,
-			"Password expired, contact your system administrator");
+			_("Password expired, contact your system administrator"));
 		    return AUTH_FATAL;
 		case PAM_ACCT_EXPIRED:
-		    log_error(NO_EXIT|NO_MAIL, "%s %s",
-			"Account expired or PAM config lacks an \"account\"",
-			"section for sudo, contact your system administrator");
+		    log_error(NO_EXIT|NO_MAIL,
+			_("Account expired or PAM config lacks an \"account\" "
+			"section for sudo, contact your system administrator"));
 		    return AUTH_FATAL;
 	    }
 	    /* FALLTHROUGH */
@@ -175,7 +173,7 @@ pam_verify(struct passwd *pw, char *prompt, sudo_auth *auth)
 	    return AUTH_FAILURE;
 	default:
 	    if ((s = pam_strerror(pamh, *pam_status)))
-		log_error(NO_EXIT|NO_MAIL, "pam_authenticate: %s", s);
+		log_error(NO_EXIT|NO_MAIL, _("pam_authenticate: %s"), s);
 	    return AUTH_FATAL;
     }
 }
@@ -293,7 +291,7 @@ converse(int num_msg, PAM_CONST struct pam_message **msg,
 		    (def_prompt[9] == ' ' && def_prompt[10] == '\0'));
 
 		/* Only override PAM prompt if it matches /^Password: ?/ */
-#if defined(PAM_TEXT_DOMAIN) && defined(HAVE_DGETTEXT)
+#if defined(PAM_TEXT_DOMAIN) && defined(HAVE_LIBINTL_H)
 		if (!def_passprompt_override && (std_prompt ||
 		    (strcmp(pm->msg, dgettext(PAM_TEXT_DOMAIN, "Password: ")) &&
 		    strcmp(pm->msg, dgettext(PAM_TEXT_DOMAIN, "Password:")))))
