@@ -78,7 +78,7 @@ securid_init(struct passwd *pw, char **promptp, sudo_auth *auth)
     if (AceInitialize() != SD_FALSE)
 	return AUTH_SUCCESS;
 
-    warningx("failed to initialise the ACE API library");
+    warningx(_("failed to initialise the ACE API library"));
     return AUTH_FATAL;
 }
 
@@ -103,7 +103,7 @@ securid_setup(struct passwd *pw, char **promptp, sudo_auth *auth)
 
     /* Re-initialize SecurID every time. */
     if (SD_Init(sd) != ACM_OK) {
-	warningx("unable to contact the SecurID server");
+	warningx(_("unable to contact the SecurID server"));
 	return AUTH_FATAL;
     }
 
@@ -112,23 +112,23 @@ securid_setup(struct passwd *pw, char **promptp, sudo_auth *auth)
 
     switch (retval) {
 	case ACM_OK:
-		warningx("User ID locked for SecurID Authentication");
+		warningx(_("User ID locked for SecurID Authentication"));
 		return AUTH_SUCCESS;
 
         case ACE_UNDEFINED_USERNAME:
-		warningx("invalid username length for SecurID");
+		warningx(_("invalid username length for SecurID"));
 		return AUTH_FATAL;
 
 	case ACE_ERR_INVALID_HANDLE:
-		warningx("invalid Authentication Handle for SecurID");
+		warningx(_("invalid Authentication Handle for SecurID"));
 		return AUTH_FATAL;
 
 	case ACM_ACCESS_DENIED:
-		warningx("SecurID communication failed");
+		warningx(_("SecurID communication failed"));
 		return AUTH_FATAL;
 
 	default:
-		warningx("unknown SecurID error");
+		warningx(_("unknown SecurID error"));
 		return AUTH_FATAL;
 	}
 }
@@ -161,17 +161,17 @@ securid_verify(struct passwd *pw, char *pass, sudo_auth *auth)
 		break;
 
 	case ACE_UNDEFINED_PASSCODE:
-		warningx("invalid passcode length for SecurID");
+		warningx(_("invalid passcode length for SecurID"));
 		rval = AUTH_FATAL;
 		break;
 
 	case ACE_UNDEFINED_USERNAME:
-		warningx("invalid username length for SecurID");
+		warningx(_("invalid username length for SecurID"));
 		rval = AUTH_FATAL;
 		break;
 
 	case ACE_ERR_INVALID_HANDLE:
-		warningx("invalid Authentication Handle for SecurID");
+		warningx(_("invalid Authentication Handle for SecurID"));
 		rval = AUTH_FATAL;
 		break;
 
@@ -210,7 +210,7 @@ then enter the new token code.\n", \
 		break;
 
 	default:
-		warningx("unknown SecurID error");
+		warningx(_("unknown SecurID error"));
 		rval = AUTH_FATAL;
 		break;
     }
