@@ -55,10 +55,10 @@ audit_sudo_selected(int sf)
 	if (getaudit_addr(&ainfo_addr, sizeof(ainfo_addr)) < 0) {
 		if (errno == ENOSYS) {
 			if (getaudit(&ainfo) < 0)
-				log_error(0, "getaudit: failed");
+				log_error(0, _("getaudit: failed"));
 			mask = &ainfo.ai_mask;
 		} else
-			log_error(0, "getaudit: failed");
+			log_error(0, _("getaudit: failed"));
         } else
 		mask = &ainfo_addr.ai_mask;
 	sorf = (sf == 0) ? AU_PRS_SUCCESS : AU_PRS_FAILURE;
@@ -84,7 +84,7 @@ bsm_audit_success(char **exec_args)
 	if (auditon(A_GETCOND, (caddr_t)&au_cond, sizeof(long)) < 0) {
 		if (errno == AUDIT_NOT_CONFIGURED)
 			return;
-		log_error(0, "Could not determine audit condition");
+		log_error(0, _("Could not determine audit condition"));
 	}
 	if (au_cond == AUC_NOAUDIT)
 		return;
@@ -95,9 +95,9 @@ bsm_audit_success(char **exec_args)
 	if (!audit_sudo_selected(0))
 		return;
 	if (getauid(&auid) < 0)
-		log_error(0, "getauid failed");
+		log_error(0, _("getauid failed"));
 	if ((aufd = au_open()) == -1)
-		log_error(0, "au_open: failed");
+		log_error(0, _("au_open: failed"));
 	if (getaudit_addr(&ainfo_addr, sizeof(ainfo_addr)) == 0) {
 		tok = au_to_subject_ex(auid, geteuid(), getegid(), getuid(),
 		    getuid(), pid, pid, &ainfo_addr.ai_termid);
@@ -106,24 +106,24 @@ bsm_audit_success(char **exec_args)
 		 * NB: We should probably watch out for ERANGE here.
 		 */
 		if (getaudit(&ainfo) < 0)
-			log_error(0, "getaudit: failed");
+			log_error(0, _("getaudit: failed"));
 		tok = au_to_subject(auid, geteuid(), getegid(), getuid(),
 		    getuid(), pid, pid, &ainfo.ai_termid);
 	} else
-		log_error(0, "getaudit: failed");
+		log_error(0, _("getaudit: failed"));
 	if (tok == NULL)
-		log_error(0, "au_to_subject: failed");
+		log_error(0, _("au_to_subject: failed"));
 	au_write(aufd, tok);
 	tok = au_to_exec_args(exec_args);
 	if (tok == NULL)
-		log_error(0, "au_to_exec_args: failed");
+		log_error(0, _("au_to_exec_args: failed"));
 	au_write(aufd, tok);
 	tok = au_to_return32(0, 0);
 	if (tok == NULL)
-		log_error(0, "au_to_return32: failed");
+		log_error(0, _("au_to_return32: failed"));
 	au_write(aufd, tok);
 	if (au_close(aufd, 1, AUE_sudo) == -1)
-		log_error(0, "unable to commit audit record");
+		log_error(0, _("unable to commit audit record"));
 }
 
 void
@@ -145,42 +145,42 @@ bsm_audit_failure(char **exec_args, char const *const fmt, va_list ap)
 	if (auditon(A_GETCOND, &au_cond, sizeof(long)) < 0) {
 		if (errno == AUDIT_NOT_CONFIGURED)
 			return;
-		log_error(0, "Could not determine audit condition");
+		log_error(0, _("Could not determine audit condition"));
 	}
 	if (au_cond == AUC_NOAUDIT)
 		return;
 	if (!audit_sudo_selected(1))
 		return;
 	if (getauid(&auid) < 0)
-		log_error(0, "getauid: failed");
+		log_error(0, _("getauid: failed"));
 	if ((aufd = au_open()) == -1)
-		log_error(0, "au_open: failed");
+		log_error(0, _("au_open: failed"));
 	if (getaudit_addr(&ainfo_addr, sizeof(ainfo_addr)) == 0) { 
 		tok = au_to_subject_ex(auid, geteuid(), getegid(), getuid(),
 		    getuid(), pid, pid, &ainfo_addr.ai_termid);
 	} else if (errno == ENOSYS) {
 		if (getaudit(&ainfo) < 0) 
-			log_error(0, "getaudit: failed");
+			log_error(0, _("getaudit: failed"));
 		tok = au_to_subject(auid, geteuid(), getegid(), getuid(),
 		    getuid(), pid, pid, &ainfo.ai_termid);
 	} else
-		log_error(0, "getaudit: failed");
+		log_error(0, _("getaudit: failed"));
 	if (tok == NULL)
-		log_error(0, "au_to_subject: failed");
+		log_error(0, _("au_to_subject: failed"));
 	au_write(aufd, tok);
 	tok = au_to_exec_args(exec_args);
 	if (tok == NULL)
-		log_error(0, "au_to_exec_args: failed");
+		log_error(0, _("au_to_exec_args: failed"));
 	au_write(aufd, tok);
 	(void) vsnprintf(text, sizeof(text), fmt, ap);
 	tok = au_to_text(text);
 	if (tok == NULL)
-		log_error(0, "au_to_text: failed");
+		log_error(0, _("au_to_text: failed"));
 	au_write(aufd, tok);
 	tok = au_to_return32(EPERM, 1);
 	if (tok == NULL)
-		log_error(0, "au_to_return32: failed");
+		log_error(0, _("au_to_return32: failed"));
 	au_write(aufd, tok);
 	if (au_close(aufd, 1, AUE_sudo) == -1)
-		log_error(0, "unable to commit audit record");
+		log_error(0, _("unable to commit audit record"));
 }
