@@ -328,7 +328,7 @@ struct sudo_ldap_handle {
     LDAP *ld;
     struct ldap_result *result;
     char *username;
-    GETGROUPS_T *groups;
+    struct group_list *grlist;
 };
 
 struct sudo_nss sudo_nss_ldap = {
@@ -360,7 +360,7 @@ sudo_ldap_conf_add_ports(void)
 
     hostbuf[0] = '\0';
     if (snprintf(defport, sizeof(defport), ":%d", ldap_conf.port) >= sizeof(defport))
-	errorx(1, "sudo_ldap_conf_add_ports: port too large");
+	errorx(1, _("sudo_ldap_conf_add_ports: port too large"));
 
     for ((host = strtok(ldap_conf.host, " \t")); host; (host = strtok(NULL, " \t"))) {
 	if (hostbuf[0] != '\0') {
@@ -383,7 +383,7 @@ sudo_ldap_conf_add_ports(void)
     return;
 
 toobig:
-    errorx(1, "sudo_ldap_conf_add_ports: out of space expanding hostbuf");
+    errorx(1, _("sudo_ldap_conf_add_ports: out of space expanding hostbuf"));
 }
 #endif
 
@@ -412,7 +412,7 @@ sudo_ldap_parse_uri(const struct ldap_config_list_str *uri_list)
 		nldaps++;
 		host = uri + 8;
 	    } else {
-		warningx("unsupported LDAP uri type: %s", uri);
+		warningx(_("unsupported LDAP uri type: %s"), uri);
 		goto done;
 	    }
 
@@ -441,17 +441,17 @@ sudo_ldap_parse_uri(const struct ldap_config_list_str *uri_list)
 	    }
 	}
 	if (hostbuf[0] == '\0') {
-	    warningx("invalid uri: %s", uri_list);
+	    warningx(_("invalid uri: %s"), uri_list);
 	    goto done;
 	}
 
 	if (nldaps != 0) {
 	    if (nldap != 0) {
-		warningx("cannot mix ldap and ldaps URIs");
+		warningx(_("unable to mix ldap and ldaps URIs"));
 		goto done;
 	    }
 	    if (ldap_conf.ssl_mode == SUDO_LDAP_STARTTLS) {
-		warningx("cannot mix ldaps and starttls");
+		warningx(_("unable to mix ldaps and starttls"));
 		goto done;
 	    }
 	    ldap_conf.ssl_mode = SUDO_LDAP_SSL;
@@ -470,7 +470,7 @@ done:
     return rc;
 
 toobig:
-    errorx(1, "sudo_ldap_parse_uri: out of space building hostbuf");
+    errorx(1, _("sudo_ldap_parse_uri: out of space building hostbuf"));
 }
 #else
 static char *
@@ -533,7 +533,7 @@ sudo_ldap_init(LDAP **ldp, const char *host, int port)
 	    rc = ldapssl_clientauth_init(ldap_conf.tls_certfile, NULL,
 		ldap_conf.tls_keyfile != NULL, ldap_conf.tls_keyfile, NULL);
 	    if (rc != LDAP_SUCCESS) {
-		warningx("unable to initialize SSL cert and key db: %s",
+		warningx(_("unable to initialize SSL cert and key db: %s"),
 		    ldapssl_err2string(rc));
 		goto done;
 	    }
@@ -929,13 +929,13 @@ sudo_ldap_timefilter(char *buffer, size_t buffersize)
     /* Make sure we have a formatted timestamp for __now__. */
     time(&now);
     if ((tp = gmtime(&now)) == NULL) {
-	warning("unable to get GMT");
+	warning(_("unable to get GMT time"));
 	goto done;
     }
 
     /* Format the timestamp according to the RFC. */
-    if (strftime(timebuffer, sizeof(timebuffer), "%Y%m%d%H%MZ", tp) == 0) {
-	warning("unable to format timestamp");
+    if (strftime(timebuffer, sizeof(timebuffer), "%Y%m%d%H%M%SZ", tp) == 0) {
+	warning(_("unable to format timestamp"));
 	goto done;
     }
 
@@ -943,7 +943,7 @@ sudo_ldap_timefilter(char *buffer, size_t buffersize)
     bytes = snprintf(buffer, buffersize, "(&(|(!(sudoNotAfter=*))(sudoNotAfter>=%s))(|(!(sudoNotBefore=*))(sudoNotBefore<=%s)))",
 	timebuffer, timebuffer);
     if (bytes < 0 || bytes >= buffersize) {
-	warning("unable to build time filter");
+	warning(_("unable to build time filter"));
 	bytes = 0;
     }
 
@@ -974,6 +974,7 @@ sudo_ldap_build_pass1(struct passwd *pw)
 {
     struct group *grp;
     char *buf, timebuffer[TIMEFILTER_LENGTH];
+    struct group_list *grlist;
     size_t sz = 0;
     int i;
 
@@ -984,17 +985,15 @@ sudo_ldap_build_pass1(struct passwd *pw)
     /* Then add (|(sudoUser=USERNAME)(sudoUser=ALL)) + NUL */
     sz += 29 + strlen(pw->pw_name);
 
-    /* Add space for groups */
+    /* Add space for primary and supplementary groups */
     if ((grp = sudo_getgrgid(pw->pw_gid)) != NULL) {
-	sz += 12 + strlen(grp->gr_name);	/* primary group */
-	gr_delref(grp);
+	sz += 12 + strlen(grp->gr_name);
     }
-    for (i = 0; i < user_ngroups; i++) {
-	if (user_groups[i] == pw->pw_gid)
-	    continue;
-	if ((grp = sudo_getgrgid(user_groups[i])) != NULL) {
-	    sz += 12 + strlen(grp->gr_name);	/* supplementary group */
-	    gr_delref(grp);
+    if ((grlist = get_group_list(pw)) != NULL) {
+	for (i = 0; i < grlist->ngroups; i++) {
+	    if (grp != NULL && strcasecmp(grlist->groups[i], grp->gr_name) == 0)
+		continue;
+	    sz += 12 + strlen(grlist->groups[i]);
 	}
     }
 
@@ -1020,28 +1019,32 @@ sudo_ldap_build_pass1(struct passwd *pw)
     (void) strlcat(buf, ")", sz);
 
     /* Append primary group */
-    if ((grp = sudo_getgrgid(pw->pw_gid)) != NULL) {
+    if (grp != NULL) {
 	(void) strlcat(buf, "(sudoUser=%", sz);
 	(void) strlcat(buf, grp->gr_name, sz);
 	(void) strlcat(buf, ")", sz);
-	gr_delref(grp);
     }
 
     /* Append supplementary groups */
-    for (i = 0; i < user_ngroups; i++) {
-	if (user_groups[i] == pw->pw_gid)
-	    continue;
-	if ((grp = sudo_getgrgid(user_groups[i])) != NULL) {
+    if (grlist != NULL) {
+	for (i = 0; i < grlist->ngroups; i++) {
+	    if (grp != NULL && strcasecmp(grlist->groups[i], grp->gr_name) == 0)
+		continue;
 	    (void) strlcat(buf, "(sudoUser=%", sz);
-	    (void) strlcat(buf, grp->gr_name, sz);
+	    (void) strlcat(buf, grlist->groups[i], sz);
 	    (void) strlcat(buf, ")", sz);
-	    gr_delref(grp);
 	}
     }
 
+    /* Done with groups. */
+    if (grlist != NULL)
+	grlist_delref(grlist);
+    if (grp != NULL)
+	gr_delref(grp);
+
     /* Add ALL to list and end the global OR */
     if (strlcat(buf, "(sudoUser=ALL)", sz) >= sz)
-	errorx(1, "sudo_ldap_build_pass1 allocation mismatch");
+	errorx(1, _("sudo_ldap_build_pass1 allocation mismatch"));
 
     /* Add the time restriction, or simply end the global OR. */
     if (ldap_conf.timed) {
@@ -1430,7 +1433,7 @@ sudo_ldap_display_defaults(struct sudo_nss *nss, struct passwd *pw,
 		else
 		    prefix = ", ";
 		for (p = bv; *p != NULL; p++) {
-		    lbuf_append(lbuf, prefix, (*p)->bv_val, NULL);
+		    lbuf_append(lbuf, "%s%s", prefix, (*p)->bv_val);
 		    prefix = ", ";
 		    count++;
 		}
@@ -1464,7 +1467,7 @@ sudo_ldap_display_entry_short(LDAP *ld, LDAPMessage *entry, struct lbuf *lbuf)
     struct berval **bv, **p;
     int count = 0;
 
-    lbuf_append(lbuf, "    (", NULL);
+    lbuf_append(lbuf, "    (");
 
     /* get the RunAsUser Values from the entry */
     bv = ldap_get_values_len(ld, entry, "sudoRunAsUser");
@@ -1472,48 +1475,39 @@ sudo_ldap_display_entry_short(LDAP *ld, LDAPMessage *entry, struct lbuf *lbuf)
 	bv = ldap_get_values_len(ld, entry, "sudoRunAs");
     if (bv != NULL) {
 	for (p = bv; *p != NULL; p++) {
-	    if (p != bv)
-		lbuf_append(lbuf, ", ", NULL);
-	    lbuf_append(lbuf, (*p)->bv_val, NULL);
+	    lbuf_append(lbuf, "%s%s", p != bv ? ", " : "", (*p)->bv_val);
 	}
 	ldap_value_free_len(bv);
     } else
-	lbuf_append(lbuf, def_runas_default, NULL);
+	lbuf_append(lbuf, "%s", def_runas_default);
 
     /* get the RunAsGroup Values from the entry */
     bv = ldap_get_values_len(ld, entry, "sudoRunAsGroup");
     if (bv != NULL) {
-	lbuf_append(lbuf, " : ", NULL);
+	lbuf_append(lbuf, " : ");
 	for (p = bv; *p != NULL; p++) {
-	    if (p != bv)
-		lbuf_append(lbuf, ", ", NULL);
-	    lbuf_append(lbuf, (*p)->bv_val, NULL);
+	    lbuf_append(lbuf, "%s%s", p != bv ? ", " : "", (*p)->bv_val);
 	}
 	ldap_value_free_len(bv);
     }
-    lbuf_append(lbuf, ") ", NULL);
+    lbuf_append(lbuf, ") ");
 
     /* get the Option Values from the entry */
     bv = ldap_get_values_len(ld, entry, "sudoOption");
     if (bv != NULL) {
-	char *cp, *tag;
-
 	for (p = bv; *p != NULL; p++) {
-	    cp = (*p)->bv_val;
+	    char *cp = (*p)->bv_val;
 	    if (*cp == '!')
 		cp++;
-	    tag = NULL;
 	    if (strcmp(cp, "authenticate") == 0)
-		tag = (*p)->bv_val[0] == '!' ?
-		    "NOPASSWD: " : "PASSWD: ";
+		lbuf_append(lbuf, (*p)->bv_val[0] == '!' ?
+		    "NOPASSWD: " : "PASSWD: ");
 	    else if (strcmp(cp, "noexec") == 0)
-		tag = (*p)->bv_val[0] == '!' ?
-		    "EXEC: " : "NOEXEC: ";
+		lbuf_append(lbuf, (*p)->bv_val[0] == '!' ?
+		    "EXEC: " : "NOEXEC: ");
 	    else if (strcmp(cp, "setenv") == 0)
-		tag = (*p)->bv_val[0] == '!' ?
-		    "NOSETENV: " : "SETENV: ";
-	    if (tag != NULL)
-		lbuf_append(lbuf, tag, NULL);
+		lbuf_append(lbuf, (*p)->bv_val[0] == '!' ?
+		    "NOSETENV: " : "SETENV: ");
 	}
 	ldap_value_free_len(bv);
     }
@@ -1522,14 +1516,12 @@ sudo_ldap_display_entry_short(LDAP *ld, LDAPMessage *entry, struct lbuf *lbuf)
     bv = ldap_get_values_len(ld, entry, "sudoCommand");
     if (bv != NULL) {
 	for (p = bv; *p != NULL; p++) {
-	    if (p != bv)
-		lbuf_append(lbuf, ", ", NULL);
-	    lbuf_append(lbuf, (*p)->bv_val, NULL);
+	    lbuf_append(lbuf, "%s%s", p != bv ? ", " : "", (*p)->bv_val);
 	    count++;
 	}
 	ldap_value_free_len(bv);
     }
-    lbuf_append(lbuf, "\n", NULL);
+    lbuf_append(lbuf, "\n");
 
     return count;
 }
@@ -1546,50 +1538,47 @@ sudo_ldap_display_entry_long(LDAP *ld, LDAPMessage *entry, struct lbuf *lbuf)
 
     /* extract the dn, only show the first rdn */
     rdn = sudo_ldap_get_first_rdn(ld, entry);
-    lbuf_append(lbuf, "\nLDAP Role: ", rdn ? rdn : "UNKNOWN", "\n", NULL);
+    if (rdn != NULL)
+	lbuf_append(lbuf, _("\nLDAP Role: %s\n"), rdn);
+    else
+	lbuf_append(lbuf, _("\nLDAP Role: UNKNOWN\n"));
     if (rdn)
 	ldap_memfree(rdn);
 
     /* get the RunAsUser Values from the entry */
-    lbuf_append(lbuf, "    RunAsUsers: ", NULL);
+    lbuf_append(lbuf, "    RunAsUsers: ");
     bv = ldap_get_values_len(ld, entry, "sudoRunAsUser");
     if (bv == NULL)
 	bv = ldap_get_values_len(ld, entry, "sudoRunAs");
     if (bv != NULL) {
 	for (p = bv; *p != NULL; p++) {
-	    if (p != bv)
-		lbuf_append(lbuf, ", ", NULL);
-	    lbuf_append(lbuf, (*p)->bv_val, NULL);
+	    lbuf_append(lbuf, "%s%s", p != bv ? ", " : "", (*p)->bv_val);
 	}
 	ldap_value_free_len(bv);
     } else
-	lbuf_append(lbuf, def_runas_default, NULL);
-    lbuf_append(lbuf, "\n", NULL);
+	lbuf_append(lbuf, "%s", def_runas_default);
+    lbuf_append(lbuf, "\n");
 
     /* get the RunAsGroup Values from the entry */
     bv = ldap_get_values_len(ld, entry, "sudoRunAsGroup");
     if (bv != NULL) {
-	lbuf_append(lbuf, "    RunAsGroups: ", NULL);
+	lbuf_append(lbuf, "    RunAsGroups: ");
 	for (p = bv; *p != NULL; p++) {
-	    if (p != bv)
-		lbuf_append(lbuf, ", ", NULL);
-	    lbuf_append(lbuf, (*p)->bv_val, NULL);
+	    lbuf_append(lbuf, "%s%s", p != bv ? ", " : "", (*p)->bv_val);
 	}
 	ldap_value_free_len(bv);
-	lbuf_append(lbuf, "\n", NULL);
+	lbuf_append(lbuf, "\n");
     }
 
     /* get the Option Values from the entry */
     bv = ldap_get_values_len(ld, entry, "sudoOption");
     if (bv != NULL) {
-	lbuf_append(lbuf, "    Options: ", NULL);
+	lbuf_append(lbuf, "    Options: ");
 	for (p = bv; *p != NULL; p++) {
-	    if (p != bv)
-		lbuf_append(lbuf, ", ", NULL);
-	    lbuf_append(lbuf, (*p)->bv_val, NULL);
+	    lbuf_append(lbuf, "%s%s", p != bv ? ", " : "", (*p)->bv_val);
 	}
 	ldap_value_free_len(bv);
-	lbuf_append(lbuf, "\n", NULL);
+	lbuf_append(lbuf, "\n");
     }
 
     /*
@@ -1599,7 +1588,7 @@ sudo_ldap_display_entry_long(LDAP *ld, LDAPMessage *entry, struct lbuf *lbuf)
     bv = ldap_get_values_len(ld, entry, "sudoOrder");
     if (bv != NULL) {
 	if (*bv != NULL) {
-	    lbuf_append(lbuf, "    Order: ", (*bv)->bv_val, "\n", NULL);
+	    lbuf_append(lbuf, _("    Order: %s\n"), (*bv)->bv_val);
 	}
 	ldap_value_free_len(bv);
     }
@@ -1607,9 +1596,9 @@ sudo_ldap_display_entry_long(LDAP *ld, LDAPMessage *entry, struct lbuf *lbuf)
     /* Get the command values from the entry. */
     bv = ldap_get_values_len(ld, entry, "sudoCommand");
     if (bv != NULL) {
-	lbuf_append(lbuf, "    Commands:\n", NULL);
+	lbuf_append(lbuf, _("    Commands:\n"));
 	for (p = bv; *p != NULL; p++) {
-	    lbuf_append(lbuf, "\t", (*p)->bv_val, "\n", NULL);
+	    lbuf_append(lbuf, "\t%s\n", (*p)->bv_val);
 	    count++;
 	}
 	ldap_value_free_len(bv);
@@ -1928,7 +1917,8 @@ sudo_ldap_bind_s(LDAP *ld)
 # endif
 	}
 	if (rc != LDAP_SUCCESS) {
-	    warningx("ldap_sasl_interactive_bind_s(): %s", ldap_err2string(rc));
+	    warningx("ldap_sasl_interactive_bind_s(): %s",
+		ldap_err2string(rc));
 	    return -1;
 	}
 	DPRINTF(("ldap_sasl_interactive_bind_s() ok"), 1);
@@ -1993,7 +1983,7 @@ sudo_ldap_open(struct sudo_nss *nss)
 #endif
 	rc = sudo_ldap_init(&ld, ldap_conf.host, ldap_conf.port);
     if (rc != LDAP_SUCCESS) {
-	warningx("unable to initialize LDAP: %s", ldap_err2string(rc));
+	warningx(_("unable to initialize LDAP: %s"), ldap_err2string(rc));
 	return -1;
     }
 
@@ -2024,7 +2014,7 @@ sudo_ldap_open(struct sudo_nss *nss)
 	}
 	DPRINTF(("ldap_start_tls_s_np() ok"), 1);
 #else
-	warningx("start_tls specified but LDAP libs do not support ldap_start_tls_s() or ldap_start_tls_s_np()");
+	warningx(_("start_tls specified but LDAP libs do not support ldap_start_tls_s() or ldap_start_tls_s_np()"));
 #endif /* !HAVE_LDAP_START_TLS_S && !HAVE_LDAP_START_TLS_S_NP */
     }
 
@@ -2037,7 +2027,7 @@ sudo_ldap_open(struct sudo_nss *nss)
     handle->ld = ld;
     handle->result = NULL;
     handle->username = NULL;
-    handle->groups = NULL;
+    handle->grlist = NULL;
     nss->handle = handle;
 
     return 0;
@@ -2069,7 +2059,7 @@ sudo_ldap_setdefs(struct sudo_nss *nss)
 	}
 	result = NULL;
 	rc = ldap_search_ext_s(ld, base->val, LDAP_SCOPE_SUBTREE,
-	    filt, NULL, 0, NULL, NULL, NULL, 0, &result);
+	    filt, NULL, 0, NULL, NULL, tvp, 0, &result);
 	if (rc == LDAP_SUCCESS && (entry = ldap_first_entry(ld, result))) {
 	    DPRINTF(("found:%s", ldap_get_dn(ld, entry)), 1);
 	    sudo_ldap_parse_options(ld, entry);
@@ -2093,7 +2083,7 @@ sudo_ldap_lookup(struct sudo_nss *nss, int ret, int pwflag)
     struct sudo_ldap_handle *handle = nss->handle;
     LDAP *ld;
     LDAPMessage *entry;
-    int i, rc, setenv_implied, matched = UNSPEC;
+    int i, rc, setenv_implied;
     struct ldap_result *lres = NULL;
 
     if (handle == NULL || handle->ld == NULL)
@@ -2108,11 +2098,12 @@ sudo_ldap_lookup(struct sudo_nss *nss, int ret, int pwflag)
      * password is required, so the order of the entries doesn't matter.
      */
     if (pwflag) {
-	DPRINTF(("perform search for pwflag %d", pwflag), 1);
 	int doauth = UNSPEC;
+	int matched = UNSPEC;
 	enum def_tuple pwcheck = 
 	    (pwflag == -1) ? never : sudo_defs_table[pwflag].sd_un.tuple;
 
+	DPRINTF(("perform search for pwflag %d", pwflag), 1);
 	for (i = 0; i < lres->nentries; i++) {
 	    entry = lres->entries[i].entry;
 	    if ((pwcheck == any && doauth != FALSE) ||
@@ -2162,7 +2153,6 @@ sudo_ldap_lookup(struct sudo_nss *nss, int ret, int pwflag)
 	if (rc != UNSPEC) {
 	    /* We have a match. */
 	    DPRINTF(("Command %sallowed", rc == TRUE ? "" : "NOT "), 1);
-	    matched = TRUE;
 	    if (rc == TRUE) {
 		DPRINTF(("LDAP entry: %p", entry), 1);
 		/* Apply entry-specific options. */
@@ -2255,7 +2245,7 @@ sudo_ldap_result_add_entry(struct ldap_result *lres, LDAPMessage *entry)
 	    DPRINTF(("order attribute raw: %s", (*bv)->bv_val), 1);
 	    order = strtod((*bv)->bv_val, &ep);
 	    if (ep == (*bv)->bv_val || *ep != '\0') {
-		warningx("invalid sudoOrder attribute: %s", (*bv)->bv_val);
+		warningx(_("invalid sudoOrder attribute: %s"), (*bv)->bv_val);
 		order = 0.0;
 	    }
 	    DPRINTF(("order attribute: %f", order), 1);
@@ -2295,7 +2285,7 @@ sudo_ldap_result_free_nss(struct sudo_nss *nss)
 	    efree(handle->username);
 	    handle->username = NULL;
 	}
-	handle->groups = NULL;
+	handle->grlist = NULL;
 	handle->result = NULL;
     }
 }
@@ -2321,7 +2311,7 @@ sudo_ldap_result_get(struct sudo_nss *nss, struct passwd *pw)
      * have to contact the LDAP server again.
      */
     if (handle->result) {
-	if (handle->groups == user_groups &&
+	if (handle->grlist == user_group_list &&
 	    strcmp(pw->pw_name, handle->username) == 0) {
 	    DPRINTF(("reusing previous result (user %s) with %d entries",
 		handle->username, handle->result->nentries), 1);
@@ -2364,7 +2354,7 @@ sudo_ldap_result_get(struct sudo_nss *nss, struct passwd *pw)
 	    }
 	    result = NULL;
 	    rc = ldap_search_ext_s(ld, base->val, LDAP_SCOPE_SUBTREE, filt,
-		NULL, 0, NULL, NULL, NULL, 0, &result);
+		NULL, 0, NULL, NULL, tvp, 0, &result);
 	    if (rc != LDAP_SUCCESS) {
 		DPRINTF(("nothing found for '%s'", filt), 1);
 		continue;
@@ -2395,7 +2385,7 @@ sudo_ldap_result_get(struct sudo_nss *nss, struct passwd *pw)
     /* Store everything in the sudo_nss handle. */
     handle->result = lres;
     handle->username = estrdup(pw->pw_name);
-    handle->groups = user_groups;
+    handle->grlist = user_group_list;
 
     return lres;
 }
