@@ -58,6 +58,7 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
+#include <stdarg.h>
 #include <ctype.h>
 #include <pwd.h>
 #include <grp.h>
@@ -70,16 +71,15 @@
 #if TIME_WITH_SYS_TIME
 # include <time.h>
 #endif
-#ifdef __STDC__
-# include <stdarg.h>
-#else
-# include <varargs.h>
+#ifdef HAVE_SETLOCALE
+# include <locale.h>
 #endif
 
 #include "sudoers.h"
 #include "interfaces.h"
 #include "parse.h"
 #include "redblack.h"
+#include "gettext.h"
 #include "sudoers_version.h"
 #include <gram.h>
 
@@ -109,8 +109,6 @@ static int print_unused(void *, void *);
 static int reparse_sudoers(char *, char *, int, int);
 static int run_command(char *, char **);
 static int visudo_printf(int msg_type, const char *fmt, ...);
-static void print_selfref(char *name, int, int, int);
-static void print_undefined(char *name, int, int, int);
 static void setup_signals(void);
 static void help(void) __attribute__((__noreturn__));
 static void usage(int);
@@ -155,6 +153,13 @@ main(int argc, char *argv[])
 #if !defined(HAVE_GETPROGNAME) && !defined(HAVE___PROGNAME)
     setprogname(argc > 0 ? argv[0] : "visudo");
 #endif
+
+#ifdef HAVE_SETLOCALE 
+    setlocale(LC_ALL, "");
+#endif
+    bindtextdomain("sudoers", LOCALEDIR); /* XXX - should have visudo domain */
+    textdomain("sudoers");
+
     if (argc < 1)
 	usage(1);
 
@@ -166,8 +171,8 @@ main(int argc, char *argv[])
     while ((ch = getopt(argc, argv, "Vcf:sq")) != -1) {
 	switch (ch) {
 	    case 'V':
-		(void) printf("%s version %s\n", getprogname(), PACKAGE_VERSION);
-		(void) printf("%s grammar version %d\n", getprogname(), SUDOERS_GRAMMAR_VERSION);
+		(void) printf(_("%s version %s\n"), getprogname(), PACKAGE_VERSION);
+		(void) printf(_("%s grammar version %d\n"), getprogname(), SUDOERS_GRAMMAR_VERSION);
 		exit(0);
 	    case 'c':
 		checkonly++;		/* check mode */
@@ -200,7 +205,7 @@ main(int argc, char *argv[])
     /* Mock up a fake sudo_user struct. */
     user_cmnd = "";
     if ((sudo_user.pw = sudo_getpwuid(getuid())) == NULL)
-	errorx(1, "you don't exist in the passwd database");
+	errorx(1, _("you do not exist in the %s database"), "passwd");
     get_hostname();
 
     /* Setup defaults data structures. */
@@ -230,7 +235,7 @@ main(int argc, char *argv[])
 	if (!sp->doedit)
 	    continue;
 	if (sp != tq_first(&sudoerslist)) {
-	    printf("press return to edit %s: ", sp->path);
+	    printf(_("press return to edit %s: "), sp->path);
 	    while ((ch = getchar()) != EOF && ch != '\n')
 		    continue;
 	}
@@ -276,7 +281,7 @@ edit_sudoers(struct sudoersfile *sp, char *editor, char *args, int lineno)
 #else
     if (stat(sp->path, &sb) == -1)
 #endif
-	error(1, "can't stat %s", sp->path);
+	error(1, _("unable to stat %s"), sp->path);
     orig_size = sb.st_size;
     mtim_get(&sb, &orig_mtim);
 
@@ -292,13 +297,13 @@ edit_sudoers(struct sudoersfile *sp, char *editor, char *args, int lineno)
 	    (void) lseek(sp->fd, (off_t)0, SEEK_SET);
 	    while ((nread = read(sp->fd, buf, sizeof(buf))) > 0)
 		if (write(tfd, buf, nread) != nread)
-		    error(1, "write error");
+		    error(1, _("write error"));
 
 	    /* Add missing newline at EOF if needed. */
 	    if (nread > 0 && buf[nread - 1] != '\n') {
 		buf[0] = '\n';
 		if (write(tfd, buf, 1) != 1)
-		    error(1, "write error");
+		    error(1, _("write error"));
 	    }
 	}
 	(void) close(tfd);
@@ -352,18 +357,18 @@ edit_sudoers(struct sudoersfile *sp, char *editor, char *args, int lineno)
 	 * Sanity checks.
 	 */
 	if (stat(sp->tpath, &sb) < 0) {
-	    warningx("cannot stat temporary file (%s), %s unchanged",
+	    warningx(_("unable to stat temporary file (%s), %s unchanged"),
 		sp->tpath, sp->path);
 	    return FALSE;
 	}
 	if (sb.st_size == 0 && orig_size != 0) {
-	    warningx("zero length temporary file (%s), %s unchanged",
+	    warningx(_("zero length temporary file (%s), %s unchanged"),
 		sp->tpath, sp->path);
 	    sp->modified = TRUE;
 	    return FALSE;
 	}
     } else {
-	warningx("editor (%s) failed, %s unchanged", editor, sp->path);
+	warningx(_("editor (%s) failed, %s unchanged"), editor, sp->path);
 	return FALSE;
     }
 
@@ -386,7 +391,7 @@ edit_sudoers(struct sudoersfile *sp, char *editor, char *args, int lineno)
     if (modified)
 	sp->modified = modified;
     else
-	warningx("%s unchanged", sp->tpath);
+	warningx(_("%s unchanged"), sp->tpath);
 
     return TRUE;
 }
@@ -410,7 +415,7 @@ reparse_sudoers(char *editor, char *args, int strict, int quiet)
 	last = tq_last(&sudoerslist);
 	fp = fopen(sp->tpath, "r+");
 	if (fp == NULL)
-	    errorx(1, "can't re-open temporary file (%s), %s unchanged.",
+	    errorx(1, _("unable to re-open temporary file (%s), %s unchanged."),
 		sp->tpath, sp->path);
 
 	/* Clean slate for each parse */
@@ -420,7 +425,7 @@ reparse_sudoers(char *editor, char *args, int strict, int quiet)
 	/* Parse the sudoers temp file */
 	yyrestart(fp);
 	if (yyparse() && !parse_error) {
-	    warningx("unabled to parse temporary file (%s), unknown error",
+	    warningx(_("unabled to parse temporary file (%s), unknown error"),
 		sp->tpath);
 	    parse_error = TRUE;
 	    errorfile = sp->path;
@@ -454,13 +459,15 @@ reparse_sudoers(char *editor, char *args, int strict, int quiet)
 		    break;
 		}
 	    }
-	    if (sp == NULL)
-		errorx(1, "internal error, can't find %s in list!", sudoers);
+	    if (sp == NULL) {
+		errorx(1, _("internal error, unable to find %s in list!"),
+		    sudoers);
+	    }
 	}
 
 	/* If any new #include directives were added, edit them too. */
 	for (sp = last->next; sp != NULL; sp = sp->next) {
-	    printf("press return to edit %s: ", sp->path);
+	    printf(_("press return to edit %s: "), sp->path);
 	    while ((ch = getchar()) != EOF && ch != '\n')
 		    continue;
 	    edit_sudoers(sp, editor, args, errorlineno);
@@ -490,23 +497,23 @@ install_sudoers(struct sudoersfile *sp, int oldperms)
 #else
 	if (stat(sp->path, &sb) == -1)
 #endif
-	    error(1, "can't stat %s", sp->path);
+	    error(1, _("unable to stat %s"), sp->path);
 	if (chown(sp->tpath, sb.st_uid, sb.st_gid) != 0) {
-	    warning("unable to set (uid, gid) of %s to (%d, %d)",
+	    warning(_("unable to set (uid, gid) of %s to (%d, %d)"),
 		sp->tpath, sb.st_uid, sb.st_gid);
 	}
 	if (chmod(sp->tpath, sb.st_mode & 0777) != 0) {
-	    warning("unable to change mode of %s to 0%o", sp->tpath,
+	    warning(_("unable to change mode of %s to 0%o"), sp->tpath,
 		(sb.st_mode & 0777));
 	}
     } else {
 	if (chown(sp->tpath, SUDOERS_UID, SUDOERS_GID) != 0) {
-	    warning("unable to set (uid, gid) of %s to (%d, %d)",
+	    warning(_("unable to set (uid, gid) of %s to (%d, %d)"),
 		sp->tpath, SUDOERS_UID, SUDOERS_GID);
 	    return FALSE;
 	}
 	if (chmod(sp->tpath, SUDOERS_MODE) != 0) {
-	    warning("unable to change mode of %s to 0%o", sp->tpath,
+	    warning(_("unable to change mode of %s to 0%o"), sp->tpath,
 		SUDOERS_MODE);
 	    return FALSE;
 	}
@@ -523,7 +530,7 @@ install_sudoers(struct sudoersfile *sp, int oldperms)
     } else {
 	if (errno == EXDEV) {
 	    char *av[4];
-	    warningx("%s and %s not on the same file system, using mv to rename",
+	    warningx(_("%s and %s not on the same file system, using mv to rename"),
 	      sp->tpath, sp->path);
 
 	    /* Build up argument vector for the command */
@@ -537,7 +544,7 @@ install_sudoers(struct sudoersfile *sp, int oldperms)
 
 	    /* And run it... */
 	    if (run_command(_PATH_MV, av)) {
-		warningx("command failed: '%s %s %s', %s unchanged",
+		warningx(_("command failed: '%s %s %s', %s unchanged"),
 		    _PATH_MV, sp->tpath, sp->path, sp->path);
 		(void) unlink(sp->tpath);
 		efree(sp->tpath);
@@ -547,7 +554,7 @@ install_sudoers(struct sudoersfile *sp, int oldperms)
 	    efree(sp->tpath);
 	    sp->tpath = NULL;
 	} else {
-	    warning("error renaming %s, %s unchanged", sp->tpath, sp->path);
+	    warning(_("error renaming %s, %s unchanged"), sp->tpath, sp->path);
 	    (void) unlink(sp->tpath);
 	    return FALSE;
 	}
@@ -607,7 +614,7 @@ whatnow(void)
     int choice, c;
 
     for (;;) {
-	(void) fputs("What now? ", stdout);
+	(void) fputs(_("What now? "), stdout);
 	choice = getchar();
 	for (c = choice; c != '\n' && c != EOF;)
 	    c = getchar();
@@ -621,10 +628,10 @@ whatnow(void)
 	    case 'Q':
 		return choice;
 	    default:
-		(void) puts("Options are:");
-		(void) puts("  (e)dit sudoers file again");
-		(void) puts("  e(x)it without saving changes to sudoers file");
-		(void) puts("  (Q)uit and save changes to sudoers file (DANGER!)\n");
+		(void) puts(_("Options are:\n"
+		    "  (e)dit sudoers file again\n"
+		    "  e(x)it without saving changes to sudoers file\n"
+		    "  (Q)uit and save changes to sudoers file (DANGER!)\n"));
 	}
     }
 }
@@ -658,14 +665,14 @@ run_command(char *path, char **argv)
 
     switch (pid = fork()) {
 	case -1:
-	    error(1, "unable to run %s", path);
+	    error(1, _("unable to execute %s"), path);
 	    break;	/* NOTREACHED */
 	case 0:
 	    sudo_endpwent();
 	    sudo_endgrent();
 	    closefrom(STDERR_FILENO + 1);
 	    execv(path, argv);
-	    warning("unable to run %s", path);
+	    warning(_("unable to run %s"), path);
 	    _exit(127);
 	    break;	/* NOTREACHED */
     }
@@ -690,13 +697,13 @@ check_syntax(char *sudoers_path, int quiet, int strict)
 	sudoers_path = "stdin";
     } else if ((yyin = fopen(sudoers_path, "r")) == NULL) {
 	if (!quiet)
-	    warning("unable to open %s", sudoers_path);
+	    warning(_("unable to open %s"), sudoers_path);
 	exit(1);
     }
     init_parser(sudoers_path, quiet);
     if (yyparse() && !parse_error) {
 	if (!quiet)
-	    warningx("failed to parse %s file, unknown error", sudoers_path);
+	    warningx(_("failed to parse %s file, unknown error"), sudoers_path);
 	parse_error = TRUE;
 	errorfile = sudoers_path;
     }
@@ -708,12 +715,12 @@ check_syntax(char *sudoers_path, int quiet, int strict)
     if (!quiet) {
 	if (parse_error) {
 	    if (errorlineno != -1)
-		(void) printf("parse error in %s near line %d\n", errorfile,
-		    errorlineno);
+		(void) printf(_("parse error in %s near line %d\n"),
+		    errorfile, errorlineno);
 	    else
-		(void) printf("parse error in %s\n", errorfile);
+		(void) printf(_("parse error in %s\n"), errorfile);
 	} else {
-	    (void) printf("%s: parsed OK\n", sudoers_path);
+	    (void) printf(_("%s: parsed OK\n"), sudoers_path);
 	}
     }
     /* Check mode and owner in strict mode. */
@@ -726,14 +733,15 @@ check_syntax(char *sudoers_path, int quiet, int strict)
 	if (sb.st_uid != SUDOERS_UID || sb.st_gid != SUDOERS_GID) {
 	    error = TRUE;
 	    if (!quiet) {
-		fprintf(stderr, "%s: wrong owner (uid, gid) should be (%d, %d)\n",
+		fprintf(stderr,
+		    _("%s: wrong owner (uid, gid) should be (%d, %d)\n"),
 		    sudoers_path, SUDOERS_UID, SUDOERS_GID);
 		}
 	}
 	if ((sb.st_mode & 07777) != SUDOERS_MODE) {
 	    error = TRUE;
 	    if (!quiet) {
-		fprintf(stderr, "%s: bad permissions, should be mode 0%o\n",
+		fprintf(stderr, _("%s: bad permissions, should be mode 0%o\n"),
 		    sudoers_path, SUDOERS_MODE);
 	    }
 	}
@@ -772,7 +780,7 @@ open_sudoers(const char *path, int doedit, int *keepopen)
 	    return NULL;
 	}
 	if (!lock_file(entry->fd, SUDO_TLOCK))
-	    errorx(1, "%s busy, try again later", entry->path);
+	    errorx(1, _("%s busy, try again later"), entry->path);
 	if ((fp = fdopen(entry->fd, "r")) == NULL)
 	    error(1, "%s", entry->path);
 	tq_append(&sudoerslist, entry);
@@ -815,7 +823,7 @@ get_editor(char **args)
 	} else {
 	    if (def_env_editor) {
 		/* If we are honoring $EDITOR this is a fatal error. */
-		errorx(1, "specified editor (%s) doesn't exist!", UserEditor);
+		errorx(1, _("specified editor (%s) doesn't exist"), UserEditor);
 	    } else {
 		/* Otherwise, just ignore $EDITOR. */
 		UserEditor = NULL;
@@ -838,7 +846,7 @@ get_editor(char **args)
 
 	if (stat(UserEditor, &user_editor_sb) != 0) {
 	    /* Should never happen since we already checked above. */
-	    error(1, "unable to stat editor (%s)", UserEditor);
+	    error(1, _("unable to stat editor (%s)"), UserEditor);
 	}
 	EditorPath = estrdup(def_editor);
 	Editor = strtok(EditorPath, ":");
@@ -886,7 +894,7 @@ get_editor(char **args)
 
 	/* Bleah, none of the editors existed! */
 	if (Editor == NULL || *Editor == '\0')
-	    errorx(1, "no editor found (editor path = %s)", def_editor);
+	    errorx(1, _("no editor found (editor path = %s)"), def_editor);
     }
     *args = EditorArgs;
     return Editor;
@@ -945,13 +953,8 @@ alias_remove_recursive(char *name, int type, int strict, int quiet)
     if ((a = alias_find(name, type)) != NULL) {
 	tq_foreach_fwd(&a->members, m) {
 	    if (m->type == ALIAS) {
-		if (strcmp(name, m->name) == 0) {
-		    print_selfref(m->name, type, strict, quiet);
+		if (!alias_remove_recursive(m->name, type, strict, quiet))
 		    error = 1;
-		} else {
-		    if (!alias_remove_recursive(m->name, type, strict, quiet))
-			error = 1;
-		}
 	    }
 	}
     }
@@ -962,6 +965,42 @@ alias_remove_recursive(char *name, int type, int strict, int quiet)
     return error;
 }
 
+static int
+check_alias(char *name, int type, int strict, int quiet)
+{
+    struct member *m;
+    struct alias *a;
+    int error = 0;
+
+    if ((a = alias_find(name, type)) != NULL) {
+	/* check alias contents */
+	tq_foreach_fwd(&a->members, m) {
+	    if (m->type == ALIAS)
+		error += check_alias(m->name, type, strict, quiet);
+	}
+    } else {
+	if (!quiet) {
+	    char *fmt;
+	    if (errno == ELOOP) {
+		fmt = strict ?
+		    _("Error: cycle in %s_Alias `%s'") :
+		    _("Warning: cycle in %s_Alias `%s'");
+	    } else {
+		fmt = strict ?
+		    _("Error: %s_Alias `%s' referenced but not defined") :
+		    _("Warning: %s_Alias `%s' referenced but not defined");
+	    }
+	    warningx(fmt,
+		type == HOSTALIAS ? "Host" : type == CMNDALIAS ? "Cmnd" :
+		type == USERALIAS ? "User" : type == RUNASALIAS ? "Runas" :
+		"Unknown", name);
+	}
+	error++;
+    }
+
+    return error;
+}
+
 /*
  * Iterate through the sudoers datastructures looking for undefined
  * aliases or unused aliases.
@@ -983,38 +1022,26 @@ check_aliases(int strict, int quiet)
 	tq_foreach_fwd(&us->users, m) {
 	    if (m->type == ALIAS) {
 		alias_seqno++;
-		if (alias_find(m->name, USERALIAS) == NULL) {
-		    print_undefined(m->name, USERALIAS, strict, quiet);
-		    error++;
-		}
+		error += check_alias(m->name, USERALIAS, strict, quiet);
 	    }
 	}
 	tq_foreach_fwd(&us->privileges, priv) {
 	    tq_foreach_fwd(&priv->hostlist, m) {
 		if (m->type == ALIAS) {
 		    alias_seqno++;
-		    if (alias_find(m->name, HOSTALIAS) == NULL) {
-			print_undefined(m->name, HOSTALIAS, strict, quiet);
-			error++;
-		    }
+		    error += check_alias(m->name, HOSTALIAS, strict, quiet);
 		}
 	    }
 	    tq_foreach_fwd(&priv->cmndlist, cs) {
 		tq_foreach_fwd(&cs->runasuserlist, m) {
 		    if (m->type == ALIAS) {
 			alias_seqno++;
-			if (alias_find(m->name, RUNASALIAS) == NULL) {
-			    print_undefined(m->name, RUNASALIAS, strict, quiet);
-			    error++;
-			}
+			error += check_alias(m->name, RUNASALIAS, strict, quiet);
 		    }
 		}
 		if ((m = cs->cmnd)->type == ALIAS) {
 		    alias_seqno++;
-		    if (alias_find(m->name, CMNDALIAS) == NULL) {
-			print_undefined(m->name, CMNDALIAS, strict, quiet);
-			error++;
-		    }
+		    error += check_alias(m->name, CMNDALIAS, strict, quiet);
 		}
 	    }
 	}
@@ -1024,28 +1051,35 @@ check_aliases(int strict, int quiet)
     tq_foreach_fwd(&userspecs, us) {
 	tq_foreach_fwd(&us->users, m) {
 	    if (m->type == ALIAS) {
+		alias_seqno++;
 		if (!alias_remove_recursive(m->name, USERALIAS, strict, quiet))
 		    error++;
 	    }
 	}
 	tq_foreach_fwd(&us->privileges, priv) {
 	    tq_foreach_fwd(&priv->hostlist, m) {
-		if (m->type == ALIAS)
+		if (m->type == ALIAS) {
+		    alias_seqno++;
 		    if (!alias_remove_recursive(m->name, HOSTALIAS, strict,
 			quiet))
 			error++;
+		}
 	    }
 	    tq_foreach_fwd(&priv->cmndlist, cs) {
 		tq_foreach_fwd(&cs->runasuserlist, m) {
-		    if (m->type == ALIAS)
+		    if (m->type == ALIAS) {
+			alias_seqno++;
 			if (!alias_remove_recursive(m->name, RUNASALIAS,
 			    strict, quiet))
 			    error++;
+		    }
 		}
-		if ((m = cs->cmnd)->type == ALIAS)
+		if ((m = cs->cmnd)->type == ALIAS) {
+		    alias_seqno++;
 		    if (!alias_remove_recursive(m->name, CMNDALIAS, strict,
 			quiet))
 			error++;
+		}
 	    }
 	}
     }
@@ -1068,9 +1102,11 @@ check_aliases(int strict, int quiet)
 	}
 	tq_foreach_fwd(&d->binding, binding) {
 	    for (m = binding; m != NULL; m = m->next) {
-		if (m->type == ALIAS)
+		if (m->type == ALIAS) {
+		    alias_seqno++;
 		    if (!alias_remove_recursive(m->name, atype, strict, quiet))
 			error++;
+		}
 	    }
 	}
     }
@@ -1083,41 +1119,13 @@ check_aliases(int strict, int quiet)
     return strict ? error : 0;
 }
 
-static void
-print_undefined(char *name, int type, int strict, int quiet)
-{
-    if (!quiet) {
-	warningx("%s: %s_Alias `%s' referenced but not defined",
-	    strict ? "Error" : "Warning",
-	    type == HOSTALIAS ? "Host" : type == CMNDALIAS ? "Cmnd" :
-	    type == USERALIAS ? "User" : type == RUNASALIAS ? "Runas" :
-	    "Unknown", name);
-    }
-}
-
-static void
-print_selfref(name, type, strict, quiet)
-    char *name;
-    int type;
-    int strict;
-    int quiet;
-{
-    if (!quiet) {
-	warningx("%s: %s_Alias `%s' references self",
-	    strict ? "Error" : "Warning",
-	    type == HOSTALIAS ? "Host" : type == CMNDALIAS ? "Cmnd" :
-	    type == USERALIAS ? "User" : type == RUNASALIAS ? "Runas" :
-	    "Unknown", name);
-    }
-}
-
 static int
 print_unused(void *v1, void *v2)
 {
     struct alias *a = (struct alias *)v1;
     char *prefix = (char *)v2;
 
-    warningx("%s: unused %s_Alias %s", prefix,
+    warningx(_("%s: unused %s_Alias %s"), prefix,
 	a->type == HOSTALIAS ? "Host" : a->type == CMNDALIAS ? "Cmnd" :
 	a->type == USERALIAS ? "User" : a->type == RUNASALIAS ? "Runas" :
 	"Unknown", a->name);
@@ -1174,15 +1182,15 @@ usage(int fatal)
 static void
 help(void)
 {
-    (void) printf("%s - safely edit the sudoers file\n\n", getprogname());
+    (void) printf(_("%s - safely edit the sudoers file\n\n"), getprogname());
     usage(0);
-    (void) puts("\nOptions:");
-    (void) puts("  -c          check-only mode");
-    (void) puts("  -f sudoers  specify sudoers file location");
-    (void) puts("  -h          display help message and exit");
-    (void) puts("  -q          less verbose (quiet) syntax error messages");
-    (void) puts("  -s          strict syntax checking");
-    (void) puts("  -V          display version information and exit");
+    (void) puts(_("\nOptions:\n"
+	"  -c          check-only mode\n"
+	"  -f sudoers  specify sudoers file location\n"
+	"  -h          display help message and exit\n"
+	"  -q          less verbose (quiet) syntax error messages\n"
+	"  -s          strict syntax checking\n"
+	"  -V          display version information and exit"));
     exit(0);
 }
 
