@@ -133,7 +133,7 @@ pty_setup(uid_t uid, const char *tty, const char *utmp_user)
     if (io_fds[SFD_USERTTY] != -1) {
 	if (!get_pty(&io_fds[SFD_MASTER], &io_fds[SFD_SLAVE],
 	    slavename, sizeof(slavename), uid))
-	    error(1, "Can't get pty");
+	    error(1, _("unable to allocate pty"));
 	/* Add entry to utmp/utmpx? */
 	if (utmp_user != NULL)
 	    utmp_login(tty, slavename, io_fds[SFD_SLAVE], utmp_user);
@@ -332,8 +332,8 @@ suspend_parent(int signo)
 	 * Only modify term if we are foreground process and either
 	 * the old tty mode was not cooked or child got SIGTT{IN,OU}
 	 */
-	sudo_debug(8, "parent is in %sground, ttymode %d -> %d",
-	    foreground ? "fore" : "back", oldmode, ttymode);
+	sudo_debug(8, "parent is in %s, ttymode %d -> %d",
+	    foreground ? "foreground" : "background", oldmode, ttymode);
 
 	if (ttymode != TERM_COOKED) {
 	    if (foreground) {
@@ -493,17 +493,21 @@ fork_pty(struct command_details *details, int sv[], int *maxfd)
 
     /*
      * Setup stdin/stdout/stderr for child, to be duped after forking.
+     * In background mode there is no stdin.
      */
-    io_fds[SFD_STDIN] = io_fds[SFD_SLAVE];
+    if (!ISSET(details->flags, CD_BACKGROUND))
+	io_fds[SFD_STDIN] = io_fds[SFD_SLAVE];
     io_fds[SFD_STDOUT] = io_fds[SFD_SLAVE];
     io_fds[SFD_STDERR] = io_fds[SFD_SLAVE];
 
-    /* Copy /dev/tty -> pty master */
     if (io_fds[SFD_USERTTY] != -1) {
-	iobufs = io_buf_new(io_fds[SFD_USERTTY], io_fds[SFD_MASTER],
-	    log_ttyin, iobufs);
+	/* Read from /dev/tty, write to pty master */
+	if (!ISSET(details->flags, CD_BACKGROUND)) {
+	    iobufs = io_buf_new(io_fds[SFD_USERTTY], io_fds[SFD_MASTER],
+		log_ttyin, iobufs);
+	}
 
-	/* Copy pty master -> /dev/tty */
+	/* Read from pty master, write to /dev/tty */
 	iobufs = io_buf_new(io_fds[SFD_MASTER], io_fds[SFD_USERTTY],
 	    log_ttyout, iobufs);
 
@@ -519,7 +523,7 @@ fork_pty(struct command_details *details, int sv[], int *maxfd)
     if (io_fds[SFD_STDIN] == -1 || !isatty(STDIN_FILENO)) {
 	pipeline = TRUE;
 	if (pipe(io_pipe[STDIN_FILENO]) != 0)
-	    error(1, "unable to create pipe");
+	    error(1, _("unable to create pipe"));
 	iobufs = io_buf_new(STDIN_FILENO, io_pipe[STDIN_FILENO][1],
 	    log_stdin, iobufs);
 	io_fds[SFD_STDIN] = io_pipe[STDIN_FILENO][0];
@@ -527,14 +531,14 @@ fork_pty(struct command_details *details, int sv[], int *maxfd)
     if (io_fds[SFD_STDOUT] == -1 || !isatty(STDOUT_FILENO)) {
 	pipeline = TRUE;
 	if (pipe(io_pipe[STDOUT_FILENO]) != 0)
-	    error(1, "unable to create pipe");
+	    error(1, _("unable to create pipe"));
 	iobufs = io_buf_new(io_pipe[STDOUT_FILENO][0], STDOUT_FILENO,
 	    log_stdout, iobufs);
 	io_fds[SFD_STDOUT] = io_pipe[STDOUT_FILENO][1];
     }
     if (io_fds[SFD_STDERR] == -1 || !isatty(STDERR_FILENO)) {
 	if (pipe(io_pipe[STDERR_FILENO]) != 0)
-	    error(1, "unable to create pipe");
+	    error(1, _("unable to create pipe"));
 	iobufs = io_buf_new(io_pipe[STDERR_FILENO][0], STDERR_FILENO,
 	    log_stderr, iobufs);
 	io_fds[SFD_STDERR] = io_pipe[STDERR_FILENO][1];
@@ -559,14 +563,14 @@ fork_pty(struct command_details *details, int sv[], int *maxfd)
 		n = term_raw(io_fds[SFD_USERTTY], 0);
 	    } while (!n && errno == EINTR);
 	    if (!n)
-		error(1, "Can't set terminal to raw mode");
+		error(1, _("unable to set terminal to raw mode"));
 	}
     }
 
     child = fork();
     switch (child) {
     case -1:
-	error(1, "fork");
+	error(1, _("unable to fork"));
 	break;
     case 0:
 	/* child */
@@ -741,7 +745,8 @@ send_status(int fd, struct command_status *cstat)
 	    n = send(fd, cstat, sizeof(*cstat), 0);
 	} while (n == -1 && errno == EINTR);
 	if (n != sizeof(*cstat)) {
-	    sudo_debug(8, "unable to send status to parent: %s", strerror(errno));
+	    sudo_debug(8, "unable to send status to parent: %s",
+		strerror(errno));
 	} else {
 	    sudo_debug(8, "sent status to parent");
 	}
@@ -771,19 +776,16 @@ handle_sigchld(int backchannel, struct command_status *cstat)
 	    cstat->type = CMD_WSTATUS;
 	    cstat->val = status;
 	    if (WIFSTOPPED(status)) {
-		sudo_debug(8, "command stopped, signal %d",
-		    WSTOPSIG(status));
+		sudo_debug(8, "command stopped, signal %d", WSTOPSIG(status));
 		do {
 		    child_pgrp = tcgetpgrp(io_fds[SFD_SLAVE]);
 		} while (child_pgrp == -1 && errno == EINTR);
 		if (send_status(backchannel, cstat) == -1)
 		    return alive; /* XXX */
 	    } else if (WIFSIGNALED(status)) {
-		sudo_debug(8, "command killed, signal %d",
-		    WTERMSIG(status));
+		sudo_debug(8, "command killed, signal %d", WTERMSIG(status));
 	    } else {
-		sudo_debug(8, "command exited: %d",
-		    WEXITSTATUS(status));
+		sudo_debug(8, "command exited: %d", WEXITSTATUS(status));
 	    }
 	}
 	if (!WIFSTOPPED(status))
@@ -821,7 +823,7 @@ exec_monitor(struct command_details *details, int backchannel)
      * the select() loop.
      */
     if (pipe_nonblock(signal_pipe) != 0)
-	error(1, "cannot create pipe");
+	error(1, _("unable to create pipe"));
 
     /* Reset SIGWINCH and SIGALRM. */
     zero_bytes(&sa, sizeof(sa));
@@ -853,7 +855,7 @@ exec_monitor(struct command_details *details, int backchannel)
     if (io_fds[SFD_SLAVE] != -1) {
 #ifdef TIOCSCTTY
 	if (ioctl(io_fds[SFD_SLAVE], TIOCSCTTY, NULL) != 0)
-	    error(1, "unable to set controlling tty");
+	    error(1, _("unable to set controlling tty"));
 #else
 	/* Set controlling tty by reopening slave. */
 	if ((n = open(slavename, O_RDWR)) >= 0)
@@ -872,10 +874,10 @@ exec_monitor(struct command_details *details, int backchannel)
 
     /* Start command and wait for it to stop or exit */
     if (pipe(errpipe) == -1)
-	error(1, "unable to create pipe");
+	error(1, _("unable to create pipe"));
     child = fork();
     if (child == -1) {
-	warning("Can't fork");
+	warning(_("unable to fork"));
 	goto bad;
     }
     if (child == 0) {
@@ -939,7 +941,7 @@ exec_monitor(struct command_details *details, int backchannel)
 		goto done;
 	    if (errno == EINTR)
 		continue;
-	    error(1, "select failed");
+	    error(1, _("select failed"));
 	}
 
 	if (FD_ISSET(signal_pipe[0], fdsr)) {
@@ -947,7 +949,7 @@ exec_monitor(struct command_details *details, int backchannel)
 	    if (n == -1) {
 		if (errno == EINTR || errno == EAGAIN)
 		    continue;
-		warning("error reading from signal pipe");
+		warning(_("error reading from signal pipe"));
 		goto done;
 	    }
 	    /*
@@ -966,7 +968,7 @@ exec_monitor(struct command_details *details, int backchannel)
 	    if (n == -1) {
 		if (errno == EINTR)
 		    continue;
-		warning("error reading from pipe");
+		warning(_("error reading from pipe"));
 		goto done;
 	    }
 	    /* Got errno or EOF, either way we are done with errpipe. */
@@ -982,11 +984,12 @@ exec_monitor(struct command_details *details, int backchannel)
 	    if (n == -1) {
 		if (errno == EINTR)
 		    continue;
-		warning("error reading from socketpair");
+		warning(_("error reading from socketpair"));
 		goto done;
 	    }
 	    if (cstmp.type != CMD_SIGNO) {
-		warningx("unexpected reply type on backchannel: %d", cstmp.type);
+		warningx(_("unexpected reply type on backchannel: %d"),
+		    cstmp.type);
 		continue;
 	    }
 	    deliver_signal(child, cstmp.val);
@@ -1071,7 +1074,7 @@ flush_output(void)
 		break; /* all I/O flushed */
 	    if (errno == EINTR)
 		continue;
-	    error(1, "select failed");
+	    error(1, _("select failed"));
 	}
 	if (perform_io(fdsr, fdsw, NULL) != 0)
 	    break;
