@@ -26,16 +26,26 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#ifdef HAVE_SETLOCALE
+# include <locale.h>
+#endif
 
 #include "missing.h"
+#include "gettext.h"
 
 int
 main (int argc, char *argv[])
 {
     char *cp, *cmnd;
 
+#ifdef HAVE_SETLOCALE 
+    setlocale(LC_ALL, "");
+#endif
+    bindtextdomain(PACKAGE_NAME, LOCALEDIR);
+    textdomain(PACKAGE_NAME);
+
     if (argc < 2)
-	errx(EXIT_FAILURE, "requires at least one argument");
+	errx(EXIT_FAILURE, _("requires at least one argument"));
 
     /* Shift argv and make a copy of the command to execute. */
     argv++;
@@ -51,6 +61,6 @@ main (int argc, char *argv[])
 	*cp = '-';
     }
     execv(cmnd, argv);
-    warn("unable to execute %s", argv[0]);
+    warn(_("unable to execute %s"), argv[0]);
     _exit(EXIT_FAILURE);
 }
