@@ -121,7 +121,6 @@ static int iolog_open(struct plugin_container *plugin, char * const settings[],
     int argc, char * const argv[], char * const user_env[]);
 static void iolog_close(struct plugin_container *plugin, int exit_status,
     int error);
-static char *escape_cmnd(const char *src);
 
 /* Policy plugin convenience functions. */
 static int policy_open(struct plugin_container *plugin, char * const settings[],
@@ -169,15 +168,17 @@ main(int argc, char *argv[], char *envp[])
     malloc_options = "AFGJPR";
 #endif
 
-#ifdef HAVE_SETLOCALE
-    setlocale(LC_ALL, "");
-#endif
-
 #if !defined(HAVE_GETPROGNAME) && !defined(HAVE___PROGNAME)
     if (argc > 0)
 	setprogname(argv[0]);
 #endif
 
+#ifdef HAVE_SETLOCALE
+    setlocale(LC_ALL, "");
+#endif
+    bindtextdomain(PACKAGE_NAME, LOCALEDIR);
+    textdomain(PACKAGE_NAME);
+
     /* Must be done before we do any password lookups */
 #if defined(HAVE_GETPRPWNAM) && defined(HAVE_SET_AUTH_PARAMETERS)
     (void) set_auth_parameters(argc, argv);
@@ -187,7 +188,7 @@ main(int argc, char *argv[], char *envp[])
 #endif /* HAVE_GETPRPWNAM && HAVE_SET_AUTH_PARAMETERS */
 
     if (geteuid() != 0)
-	errorx(1, "must be setuid root");
+	errorx(1, _("must be setuid root"));
 
     /* Reset signal mask, disable core dumps and make sure fds 0-2 are open. */
     (void) sigemptyset(&mask);
@@ -205,14 +206,14 @@ main(int argc, char *argv[], char *envp[])
 
     /* Print sudo version early, in case of plugin init failure. */
     if (ISSET(sudo_mode, MODE_VERSION)) {
-	printf("Sudo version %s\n", PACKAGE_VERSION);
+	printf(_("Sudo version %s\n"), PACKAGE_VERSION);
 	if (user_details.uid == ROOT_UID)
-	    (void) printf("Configure args: %s\n", CONFIGURE_ARGS);
+	    (void) printf(_("Configure options: %s\n"), CONFIGURE_ARGS);
     }
 
     /* Read sudo.conf and load plugins. */
     if (!sudo_load_plugins(_PATH_SUDO_CONF, &policy_plugin, &io_plugins))
-	errorx(1, "fatal error, unable to load plugins");
+	errorx(1, _("fatal error, unable to load plugins"));
 
     /* Open policy plugin. */
     ok = policy_open(&policy_plugin, settings, user_info, envp);
@@ -220,7 +221,7 @@ main(int argc, char *argv[], char *envp[])
 	if (ok == -2)
 	    usage(1);
 	else
-	    errorx(1, "unable to initialize policy plugin");
+	    errorx(1, _("unable to initialize policy plugin"));
     }
 
     switch (sudo_mode & MODE_MASK) {
@@ -275,7 +276,8 @@ main(int argc, char *argv[], char *envp[])
 		    usage(1);
 		    break;
 		default:
-		    errorx(1, "error initializing I/O plugin %s", plugin->name);
+		    errorx(1, _("error initializing I/O plugin %s"),
+			plugin->name);
 		}
 	    }
 	    command_info_to_details(command_info, &command_details);
@@ -290,18 +292,12 @@ main(int argc, char *argv[], char *envp[])
 	    if (ISSET(command_details.flags, CD_SUDOEDIT)) {
 		exitcode = sudo_edit(&command_details);
 	    } else {
-		if (ISSET(sudo_mode, MODE_SHELL)) {
-		    /* Escape meta chars if running a shell with args. */
-		    if (argv_out[1] != NULL && strcmp(argv_out[1], "-c") == 0 &&
-			argv_out[2] != NULL && argv_out[3] == NULL)
-			argv_out[2] = escape_cmnd(argv_out[2]);
-		}
 		exitcode = run_command(&command_details);
 	    }
 	    /* The close method was called by sudo_edit/run_command. */
 	    break;
 	default:
-	    errorx(1, "unexpected sudo mode 0x%x", sudo_mode);
+	    errorx(1, _("unexpected sudo mode 0x%x"), sudo_mode);
     }
     exit(exitcode);
 }
@@ -324,7 +320,7 @@ fix_fds(void)
     miss[STDERR_FILENO] = fcntl(STDERR_FILENO, F_GETFL, 0) == -1;
     if (miss[STDIN_FILENO] || miss[STDOUT_FILENO] || miss[STDERR_FILENO]) {
 	if ((devnull = open(_PATH_DEVNULL, O_RDWR, 0644)) == -1)
-	    error(1, "unable to open %s", _PATH_DEVNULL);
+	    error(1, _("unable to open %s"), _PATH_DEVNULL);
 	if (miss[STDIN_FILENO] && dup2(devnull, STDIN_FILENO) == -1)
 	    error(1, "dup2");
 	if (miss[STDOUT_FILENO] && dup2(devnull, STDOUT_FILENO) == -1)
@@ -336,21 +332,66 @@ fix_fds(void)
     }
 }
 
+/*
+ * Allocate space for groups and fill in using getgrouplist()
+ * for when we cannot use getgroups().
+ */
+static int
+fill_group_list(struct user_details *ud)
+{
+    int maxgroups, tries, rval = -1;
+
+#if defined(HAVE_SYSCONF) && defined(_SC_NGROUPS_MAX)
+    maxgroups = (int)sysconf(_SC_NGROUPS_MAX);
+    if (maxgroups < 0)
+#endif
+	maxgroups = NGROUPS_MAX;
+
+    /*
+     * It is possible to belong to more groups in the group database
+     * than NGROUPS_MAX.  We start off with NGROUPS_MAX * 2 entries
+     * and double this as needed.
+     */
+    ud->groups = NULL;
+    ud->ngroups = maxgroups;
+    for (tries = 0; tries < 10 && rval == -1; tries++) {
+	ud->ngroups *= 2;
+	efree(ud->groups);
+	ud->groups = emalloc2(ud->ngroups, sizeof(GETGROUPS_T));
+	rval = getgrouplist(ud->username, ud->gid, ud->groups, &ud->ngroups);
+    }
+    return rval;
+}
+
 static char *
 get_user_groups(struct user_details *ud)
 {
-    char *gid_list = NULL;
-#ifdef HAVE_GETGROUPS
+    char *cp, *gid_list = NULL;
     size_t glsize;
-    char *cp;
     int i, len;
 
-    if ((ud->ngroups = getgroups(0, NULL)) <= 0)
-	return NULL;
+    /*
+     * Systems with mbr_check_membership() support more than NGROUPS_MAX
+     * groups so we cannot use getgroups().
+     */
+    ud->groups = NULL;
+#ifndef HAVE_MBR_CHECK_MEMBERSHIP
+    if ((ud->ngroups = getgroups(0, NULL)) > 0) {
+	ud->groups = emalloc2(ud->ngroups, sizeof(GETGROUPS_T));
+	if (getgroups(ud->ngroups, ud->groups) < 0) {
+	    efree(ud->groups);
+	    ud->groups = NULL;
+	}
+    }
+#endif /* HAVE_MBR_CHECK_MEMBERSHIP */
+    if (ud->groups == NULL) {
+	if (fill_group_list(ud) == -1)
+	    error(1, _("unable to get group vector"));
+    }
 
-    ud->groups = emalloc2(ud->ngroups, sizeof(GETGROUPS_T));
-    if (getgroups(ud->ngroups, ud->groups) < 0)
-	error(1, "can't get group vector");
+    /*
+     * Format group list as a comma-separated string of gids.
+     */
     glsize = sizeof("groups=") - 1 + (ud->ngroups * (MAX_UID_T_LEN + 1));
     gid_list = emalloc(glsize);
     memcpy(gid_list, "groups=", sizeof("groups=") - 1);
@@ -361,7 +402,6 @@ get_user_groups(struct user_details *ud)
 	    i ? "," : "", (unsigned int)ud->groups[i]);
 	cp += len;
     }
-#endif
     return gid_list;
 }
 
@@ -388,11 +428,11 @@ get_user_info(struct user_details *ud)
 
     pw = getpwuid(ud->uid);
     if (pw == NULL)
-	errorx(1, "unknown uid %u: who are you?", (unsigned int)ud->uid);
+	errorx(1, _("unknown uid %u: who are you?"), (unsigned int)ud->uid);
 
     user_info[i] = fmt_string("user", pw->pw_name);
     if (user_info[i] == NULL)
-	errorx(1, "unable to allocate memory");
+	errorx(1, _("unable to allocate memory"));
     ud->username = user_info[i] + sizeof("user=") - 1;
 
     /* Stash user's shell for use with the -s flag; don't pass to plugin. */
@@ -412,7 +452,7 @@ get_user_info(struct user_details *ud)
     if (getcwd(cwd, sizeof(cwd)) != NULL) {
 	user_info[++i] = fmt_string("cwd", cwd);
 	if (user_info[i] == NULL)
-	    errorx(1, "unable to allocate memory");
+	    errorx(1, _("unable to allocate memory"));
 	ud->cwd = user_info[i] + sizeof("cwd=") - 1;
     }
 
@@ -420,7 +460,7 @@ get_user_info(struct user_details *ud)
 	(cp = ttyname(STDERR_FILENO))) {
 	user_info[++i] = fmt_string("tty", cp);
 	if (user_info[i] == NULL)
-	    errorx(1, "unable to allocate memory");
+	    errorx(1, _("unable to allocate memory"));
 	ud->tty = user_info[i] + sizeof("tty=") - 1;
     }
 
@@ -430,7 +470,7 @@ get_user_info(struct user_details *ud)
 	strlcpy(host, "localhost", sizeof(host));
     user_info[++i] = fmt_string("host", host);
     if (user_info[i] == NULL)
-	errorx(1, "unable to allocate memory");
+	errorx(1, _("unable to allocate memory"));
     ud->host = user_info[i] + sizeof("host=") - 1;
 
     get_ttysize(&ud->ts_lines, &ud->ts_cols);
@@ -509,11 +549,13 @@ command_info_to_details(char * const info[], struct command_details *details)
 			SET(details->flags, CD_NOEXEC);
 		    break;
 		}
+#ifdef _PATH_SUDO_NOEXEC
 		/* XXX - deprecated */
 		if (strncmp("noexec_file=", info[i], sizeof("noexec_file=") - 1) == 0) {
 		    noexec_path = info[i] + sizeof("noexec_file=") - 1;
 		    break;
 		}
+#endif /* _PATH_SUDO_NOEXEC */
 		break;
 	    case 'p':
 		if (strncmp("preserve_groups=", info[i], sizeof("preserve_groups=") - 1) == 0) {
@@ -728,39 +770,39 @@ set_project(struct passwd *pw)
 	case SETPROJ_ERR_TASK:
 	    switch (errno) {
 	    case EAGAIN:
-		warningx("resource control limit has been reached");
+		warningx(_("resource control limit has been reached"));
 		break;
 	    case ESRCH:
-		warningx("user \"%s\" is not a member of project \"%s\"",
+		warningx(_("user \"%s\" is not a member of project \"%s\""),
 		    pw->pw_name, proj.pj_name);
 		break;
 	    case EACCES:
-		warningx("the invoking task is final");
+		warningx(_("the invoking task is final"));
 		break;
 	    default:
-		warningx("could not join project \"%s\"", proj.pj_name);
+		warningx(_("could not join project \"%s\""), proj.pj_name);
 	    }
 	case SETPROJ_ERR_POOL:
 	    switch (errno) {
 	    case EACCES:
-		warningx("no resource pool accepting default bindings "
-		    "exists for project \"%s\"", proj.pj_name);
+		warningx(_("no resource pool accepting default bindings "
+		    "exists for project \"%s\""), proj.pj_name);
 		break;
 	    case ESRCH:
-		warningx("specified resource pool does not exist for "
-		    "project \"%s\"", proj.pj_name);
+		warningx(_("specified resource pool does not exist for "
+		    "project \"%s\""), proj.pj_name);
 		break;
 	    default:
-		warningx("could not bind to default resource pool for "
-		    "project \"%s\"", proj.pj_name);
+		warningx(_("could not bind to default resource pool for "
+		    "project \"%s\""), proj.pj_name);
 	    }
 	    break;
 	default:
 	    if (errval <= 0) {
-		warningx("setproject failed for project \"%s\"", proj.pj_name);
+		warningx(_("setproject failed for project \"%s\""), proj.pj_name);
 	    } else {
-		warningx("warning, resource control assignment failed for "
-		    "project \"%s\"", proj.pj_name);
+		warningx(_("warning, resource control assignment failed for "
+		    "project \"%s\""), proj.pj_name);
 	    }
 	}
     } else {
@@ -778,16 +820,19 @@ set_project(struct passwd *pw)
 static void
 disable_execute(struct command_details *details)
 {
+#ifdef _PATH_SUDO_NOEXEC
     char *cp, **ev, **nenvp;
     int env_len = 0, env_size = 128;
+#endif /* _PATH_SUDO_NOEXEC */
 
 #ifdef HAVE_PRIV_SET
     /* Solaris privileges, remove PRIV_PROC_EXEC post-execve. */
     if (priv_set(PRIV_OFF, PRIV_LIMIT, "PRIV_PROC_EXEC", NULL) == 0)
 	return;
-    warning("unable to remove PRIV_PROC_EXEC from PRIV_LIMIT");
+    warning(_("unable to remove PRIV_PROC_EXEC from PRIV_LIMIT"));
 #endif /* HAVE_PRIV_SET */
 
+#ifdef _PATH_SUDO_NOEXEC
     nenvp = emalloc2(env_size, sizeof(char *));
     for (ev = details->envp; *ev != NULL; ev++) {
 	if (env_len + 2 > env_size) {
@@ -798,21 +843,21 @@ disable_execute(struct command_details *details)
 	 * Prune out existing preloaded libraries.
 	 * XXX - should save and append instead of replacing.
 	 */
-#if defined(__darwin__) || defined(__APPLE__)
+# if defined(__darwin__) || defined(__APPLE__)
 	if (strncmp(*ev, "DYLD_INSERT_LIBRARIES=", sizeof("DYLD_INSERT_LIBRARIES=") - 1) == 0)
 	    continue;
 	if (strncmp(*ev, "DYLD_FORCE_FLAT_NAMESPACE=", sizeof("DYLD_INSERT_LIBRARIES=") - 1) == 0)
 	    continue;
-#elif defined(__osf__) || defined(__sgi)
+# elif defined(__osf__) || defined(__sgi)
 	if (strncmp(*ev, "_RLD_LIST=", sizeof("_RLD_LIST=") - 1) == 0)
 	    continue;
-#elif defined(_AIX)
+# elif defined(_AIX)
 	if (strncmp(*ev, "LDR_PRELOAD=", sizeof("LDR_PRELOAD=") - 1) == 0)
 	    continue;
-#else
+# else
 	if (strncmp(*ev, "LD_PRELOAD=", sizeof("LD_PRELOAD=") - 1) == 0)
 	    continue;
-#endif
+# endif
 	nenvp[env_len++] = *ev;
     }
 
@@ -821,22 +866,23 @@ disable_execute(struct command_details *details)
      * http://www.fortran-2000.com/ArnaudRecipes/sharedlib.html
      * XXX - need to support 32-bit and 64-bit variants
      */
-#if defined(__darwin__) || defined(__APPLE__)
+# if defined(__darwin__) || defined(__APPLE__)
     nenvp[env_len++] = "DYLD_FORCE_FLAT_NAMESPACE=";
     cp = fmt_string("DYLD_INSERT_LIBRARIES", noexec_path);
-#elif defined(__osf__) || defined(__sgi)
+# elif defined(__osf__) || defined(__sgi)
     easprintf(&cp, "_RLD_LIST=%s:DEFAULT", noexec_path);
-#elif defined(_AIX)
+# elif defined(_AIX)
     cp = fmt_string("LDR_PRELOAD", noexec_path);
-#else
+# else
     cp = fmt_string("LD_PRELOAD", noexec_path);
-#endif
+# endif
     if (cp == NULL)
 	error(1, NULL);
     nenvp[env_len++] = cp;
     nenvp[env_len] = NULL;
 
     details->envp = nenvp;
+#endif /* _PATH_SUDO_NOEXEC */
 }
 
 /*
@@ -889,17 +935,17 @@ exec_setup(struct command_details *details, const char *ptyname, int ptyfd)
 	     */
 	    lc = login_getclass((char *)details->login_class);
 	    if (!lc) {
-		warningx("unknown login class %s", details->login_class);
+		warningx(_("unknown login class %s"), details->login_class);
 		errno = ENOENT;
 		goto done;
 	    }
 	    flags = LOGIN_SETRESOURCES|LOGIN_SETPRIORITY;
 	    if (setusercontext(lc, pw, pw->pw_uid, flags)) {
 		if (pw->pw_uid != ROOT_UID) {
-		    warning("unable to set user context");
+		    warning(_("unable to set user context"));
 		    goto done;
 		} else
-		    warning("unable to set user context");
+		    warning(_("unable to set user context"));
 	    }
 	}
 #endif /* HAVE_LOGIN_CAP_H */
@@ -910,34 +956,27 @@ exec_setup(struct command_details *details, const char *ptyname, int ptyfd)
      */
 #ifdef HAVE_SETEUID
     if (ISSET(details->flags, CD_SET_EGID) && setegid(details->egid)) {
-	warning("unable to set egid to runas gid %u", details->egid);
+	warning(_("unable to set effective gid to runas gid %u"), details->egid);
 	goto done;
     }
 #endif
     if (ISSET(details->flags, CD_SET_GID) && setgid(details->gid)) {
-	warning("unable to set gid to runas gid %u", details->gid);
+	warning(_("unable to set gid to runas gid %u"), details->gid);
 	goto done;
     }
 
     if (!ISSET(details->flags, CD_PRESERVE_GROUPS)) {
-#ifdef HAVE_GETGROUPS
 	if (details->ngroups >= 0) {
-	    if (setgroups(details->ngroups, details->groups) < 0) {
-		warning("unable to set supplementary group IDs");
+	    if (sudo_setgroups(details->ngroups, details->groups) < 0) {
+		warning(_("unable to set supplementary group IDs"));
 		goto done;
 	    }
 	}
-#else
-	if (pw && initgroups(pw->pw_name, pw->pw_gid) < 0) {
-	    warning("unable to set supplementary group IDs");
-	    goto done;
-	}
-#endif
     }
 
     if (ISSET(details->flags, CD_SET_PRIORITY)) {
 	if (setpriority(PRIO_PROCESS, 0, details->priority) != 0) {
-	    warning("unable to set process priority");
+	    warning(_("unable to set process priority"));
 	    goto done;
 	}
     }
@@ -945,7 +984,7 @@ exec_setup(struct command_details *details, const char *ptyname, int ptyfd)
 	(void) umask(details->umask);
     if (details->chroot) {
 	if (chroot(details->chroot) != 0 || chdir("/") != 0) {
-	    warning("unable to change root to %s", details->chroot);
+	    warning(_("unable to change root to %s"), details->chroot);
 	    goto done;
 	}
     }
@@ -955,19 +994,19 @@ exec_setup(struct command_details *details, const char *ptyname, int ptyfd)
 
 #ifdef HAVE_SETRESUID
     if (setresuid(details->uid, details->euid, details->euid) != 0) {
-	warning("unable to change to runas uid (%u, %u)", details->uid,
+	warning(_("unable to change to runas uid (%u, %u)"), details->uid,
 	    details->euid);
 	goto done;
     }
 #elif HAVE_SETREUID
     if (setreuid(details->uid, details->euid) != 0) {
-	warning("unable to change to runas uid (%u, %u)", details->uid,
+	warning(_("unable to change to runas uid (%u, %u)"), details->uid,
 	    details->euid);
 	goto done;
     }
 #else
     if (seteuid(details->euid) != 0 || setuid(details->euid) != 0) {
-	warning("unable to change to runas uid (%u, %u)", details->uid,
+	warning(_("unable to change to runas uid (%u, %u)"), details->uid,
 	    details->euid);
 	goto done;
     }
@@ -981,7 +1020,7 @@ exec_setup(struct command_details *details, const char *ptyname, int ptyfd)
 	if (details->chroot || strcmp(details->cwd, user_details.cwd) != 0) {
 	    /* Note: cwd is relative to the new root, if any. */
 	    if (chdir(details->cwd) != 0) {
-		warning("unable to change directory to %s", details->cwd);
+		warning(_("unable to change directory to %s"), details->cwd);
 		goto done;
 	    }
 	}
@@ -1008,30 +1047,6 @@ done:
     return rval;
 }
 
-/*
- * Escape any non-alpha numeric or blank characters to make sure
- * they are not interpreted specially by the shell.
- */
-static char *
-escape_cmnd(const char *src)
-{
-    char *cmnd, *dst;
-
-    /* Worst case scenario, we have to escape everything. */
-    cmnd = dst = emalloc((2 * strlen(src)) + 1);
-    while (*src != '\0') {
-	if (!isalnum((unsigned char)*src) && !isspace((unsigned char)*src) &&
-	    *src != '_' && *src != '-') {
-	    /* quote potential meta character */
-	    *dst++ = '\\';
-	}
-	*dst++ = *src++;
-    }
-    *dst++ = '\0';
-
-    return cmnd;
-}
-
 /*
  * Run the command and wait for it to complete.
  */
@@ -1072,7 +1087,7 @@ run_command(struct command_details *details)
 	    exitcode = WTERMSIG(cstat.val) | 128;
 	break;
     default:
-	warningx("unexpected child termination condition: %d", cstat.type);
+	warningx(_("unexpected child termination condition: %d"), cstat.type);
 	break;
     }
     return exitcode;
@@ -1112,7 +1127,7 @@ policy_list(struct plugin_container *plugin, int argc, char * const argv[],
     int verbose, const char *list_user)
 {
     if (plugin->u.policy->list == NULL) {
-	warningx("policy plugin %s does not support listing privileges",
+	warningx(_("policy plugin %s does not support listing privileges"),
 	    plugin->name);
 	return FALSE;
     }
@@ -1123,7 +1138,7 @@ static int
 policy_validate(struct plugin_container *plugin)
 {
     if (plugin->u.policy->validate == NULL) {
-	warningx("policy plugin %s does not support the -v flag",
+	warningx(_("policy plugin %s does not support the -v option"),
 	    plugin->name);
 	return FALSE;
     }
@@ -1134,7 +1149,7 @@ static void
 policy_invalidate(struct plugin_container *plugin, int remove)
 {
     if (plugin->u.policy->invalidate == NULL) {
-	errorx(1, "policy plugin %s does not support the -k/-K flags",
+	errorx(1, _("policy plugin %s does not support the -k/-K options"),
 	    plugin->name);
     }
     plugin->u.policy->invalidate(remove);
