@@ -25,13 +25,14 @@
 #define _SUDO_SUDO_H
 
 #include <limits.h>
-
 #include <pathnames.h>
+
 #include "missing.h"
 #include "alloc.h"
 #include "error.h"
 #include "fileops.h"
 #include "list.h"
+#include "gettext.h"
 
 #ifdef __TANDEM
 # define ROOT_UID       65535
@@ -226,6 +227,9 @@ void aix_setauthdb(char *user);
 /* interfaces.c */
 int get_net_ifs(char **addrinfo);
 
+/* setgroups.c */
+int sudo_setgroups(int ngids, const GETGROUPS_T *gids);
+
 #ifndef errno
 extern int errno;
 #endif
